Index: FL/Fl_Menu_.H
===================================================================
--- FL/Fl_Menu_.H	(revision 10621)
+++ FL/Fl_Menu_.H	(working copy)
@@ -176,6 +176,7 @@
   Fl_Color down_color() const {return selection_color();}
   /** For back compatibility, same as selection_color() */
   void down_color(unsigned c) {selection_color(c);}
+  void setonly(Fl_Menu_Item* item);
 };
 
 #endif
Index: src/Fl_Menu_.cxx
===================================================================
--- src/Fl_Menu_.cxx	(revision 10621)
+++ src/Fl_Menu_.cxx	(working copy)
@@ -254,8 +254,9 @@
   if (v) {
     if (v->radio()) {
       if (!v->value()) { // they are turning on a radio item
-	set_changed();
-	((Fl_Menu_Item*)v)->setonly();
+        set_changed();
+        //((Fl_Menu_Item*)v)->setonly();
+        setonly((Fl_Menu_Item*)v);
       }
       redraw();
     } else if (v->flags & FL_MENU_TOGGLE) {
@@ -276,7 +277,55 @@
   return v;
 }
 
-/** Turns the radio item "on" for the menu item and turns off adjacent radio items set. */
+/* Returns pointer to first Fl_Menu_Item of the submenu containing item, within a menu that starts at start.
+ Returns NULL if item is not present.
+ FL_SUBMENU_POINTER is properly handled.
+ If pnext is non-NULL and return value is NULL, *pnext is set upon return to last item of the start menu.
+ */
+static Fl_Menu_Item *first_submenu_item(Fl_Menu_Item *item, Fl_Menu_Item *start, Fl_Menu_Item **pnext = NULL)
+{
+  Fl_Menu_Item *j = start;
+  while (j->label()) {
+    if (j == item) return start;
+    if (j->submenu()) {
+      Fl_Menu_Item *first;
+      if (j->flags&FL_SUBMENU_POINTER) {
+        first = first_submenu_item(item, (Fl_Menu_Item*)j->user_data());
+      } else {
+        first = first_submenu_item(item, j + 1, &j);
+      }
+      if (first) return first;
+    }
+    j++;
+  }
+  if (pnext) *pnext = j;
+  return NULL;
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items of the same group. */
+void Fl_Menu_::setonly(Fl_Menu_Item* item) {
+  // find the first item of the (sub)menu containing item
+  Fl_Menu_Item* first = first_submenu_item(item, menu_);
+  if (!first) return; // item does not belong to our menu
+  item->flags |= FL_MENU_RADIO | FL_MENU_VALUE;
+  Fl_Menu_Item* j;
+  for (j = item; ; ) {	// go down
+    if (j->flags & FL_MENU_DIVIDER) break; // stop on divider lines
+    j++;
+    if (!j->text || !j->radio()) break; // stop after group
+    j->clear();
+  }
+  for (j = item-1; j>=first; j--) { // go up
+    //DEBUG printf("GO UP: WORKING ON: item='%s', flags=%x\n", j->text, j->flags);
+    if (!j->text || (j->flags&FL_MENU_DIVIDER) || !j->radio()) break;
+    j->clear();
+  }
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items set.
+ Deprecated: this method is dangerous if radio items are first in the menu.
+ Use Fl_Menu_::setonly(Fl_Menu_Item*) instead.
+ */
 void Fl_Menu_Item::setonly() {
   flags |= FL_MENU_RADIO | FL_MENU_VALUE;
   Fl_Menu_Item* j;
